<?php
require_once (__FILE__ . "/../" . "include.php");

class ORMapperQuery {

	function getTableName($object) {
		return "loworks_store_" . get_class($object);
	}

	function getCreateTableQuery($object) {
		$query = new Query(ORMapperQuery::getTableName($object));
		$vars = get_object_vars($object);
		foreach ($vars as $varsName => $varsValue) {
			$createDifinition = ORMapperQuery::getColumnType($varsName);
			if (FieldLibrary::isRequiredField($varsName)) {
				$createDifinition .= " NOT NULL";
			}
			$query->addColumn($varsName, $createDifinition);
		}
		return $query->getCreateTableQuery();
	}
	
	function getUpdateQuery($object, $whereQuery = null) {
		$query = new Query(ORMapperQuery::getTableName($object));
		$vars = get_object_vars($object);
		foreach ($vars as $varsName => $varsValue) {
			if (isset($varsValue)) {
				$query->addUpdateValue($varsName, ORMapperQuery::convertFieldValue($varsName, $varsValue));
			}
		}
		if (isset($whereQuery)) {
			$query->setWhereQuery($whereQuery->getWhereQueryObject());
		}
		else {
			$whereQuery = new ORMapperQueryWhere($object);
			$whereQuery->addEqualsAnd("id", $object->id);
			$query->setWhereQuery($whereQuery->getWhereQueryObject());
		}
		return $query->getUpdateQuery();
	}
	
	function getInsertQuery($object) {
		$query = new Query(ORMapperQuery::getTableName($object));
		$vars = get_object_vars($object);
		foreach ($vars as $varsName => $varsValue) {
			$query->addInsertValue($varsName, ORMapperQuery::convertFieldValue($varsName, $varsValue));
		}
		return $query->getInsertQuery();
	}
	
	function getDeleteQuery($object, $whereQuery = null) {
		$query = new Query(ORMapperQuery::getTableName($object));
		if (isset($whereQuery)) {
			$query->setWhereQuery($whereQuery->getWhereQueryObject());
		}
		else if (isset($object->id)) {
			$whereQuery = new ORMapperQueryWhere($object);
			$whereQuery->addEqualsAnd("id", $object->id);
			$query->setWhereQuery($whereQuery->getWhereQueryObject());
		}
		return $query->getDeleteQuery();
	}
	
	function getSelectQuery($object,
		$whereQuery,
		$orderColumnName,
		$isOrderReverse = false,
		$limit,
		$offset = 0) {
		$query = new Query(ORMapperQuery::getTableName($object));
		if (isset($whereQuery)) {
			$query->setWhereQuery($whereQuery->getWhereQueryObject());
		}
		if (isset($orderColumnName)) {
			$query->setOrderBy($orderColumnName, $isOrderReverse);
		}
		if (isset($limit)) {
			$query->setLimit($limit, $offset);
		}
		return $query->getSelectQuery();
	}
	
	function convertFieldValue($fieldName, $fieldValue) {
		return FieldLibrary::isString($fieldName) ?
			Query::getQueryStringValue($fieldValue) :
			(FieldLibrary::isTimestamp($fieldName) ?
			Query::getDateString($fieldValue) :
			$fieldValue);
	}

	function getColumnType($columnName) {
		$columnTypeString = $columnName == "id" ?
			$columnName :
			substr($columnName, 0, 1);
		$COLUMN_TYPE_MAP = array(
			"id" => "bigint(20) not NULL primary key AUTO_INCREMENT", 
			"s" => "varchar(255)",
			"d" => "bigint(20)",
			"i" => "int(11)",
			"b" => "tinyint(1)",
			"t" => "timestamp");
		return $COLUMN_TYPE_MAP[$columnTypeString];
	}
}
?>